@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2021-2024 Sébastien Helleu <flashcode@flashtux.org>
+ * Copyright (C) 2021-2025 Sébastien Helleu <flashcode@flashtux.org>
  *
  * This file is part of WeeChat, the extensible chat client.
  *
@@ -31,8 +31,8 @@ extern struct t_signal signal_list[];
 extern int signal_search_number (int signal_number);
 extern int signal_search_name (const char *name);
 extern void signal_catch (int signum, void (*handler)(int));
-extern void signal_handle ();
-extern void signal_suspend ();
-extern void signal_init ();
+extern void signal_handle (void);
+extern void signal_suspend (void);
+extern void signal_init (void);
 
 #endif /* WEECHAT_SIGNAL_H */
